@@ -41,8 +41,13 @@
 #include <gconf/gconf-client.h>
 #include "file-utils.h"
 #include "utf8-fnmatch.h"
+#include "main.h"
 
 
+#ifndef HAVE_MKDTEMP
+#include "mkdtemp.h"
+#endif                                        
+                                            
 #define BUF_SIZE 4096
 #define MAX_PATTERNS 128
 
@@ -549,87 +554,6 @@ shell_escape (const char *filename)
 }
 
 
-static gchar *
-get_terminal ()
-{
-	GConfClient *client;
-	gchar *result;
-	gchar *terminal = NULL;
-	gchar *exec_flag = NULL;
-
-	client = gconf_client_get_default ();
-	terminal = gconf_client_get_string (client, "/desktop/gnome/applications/terminal/exec", NULL);
-	g_object_unref (G_OBJECT (client));
-	
-	if (terminal) 
-		exec_flag = gconf_client_get_string (client, "/desktop/gnome/applications/terminal/exec_arg", NULL);
-
-	if (terminal == NULL) {
-		char *check;
-
-		check = g_find_program_in_path ("gnome-terminal");
-		if (check != NULL) {
-			terminal = check;
-			/* Note that gnome-terminal takes -x and
-			 * as -e in gnome-terminal is broken we use that. */
-			exec_flag = g_strdup ("-x");
-		} else {
-			if (check == NULL)
-				check = g_find_program_in_path ("nxterm");
-			if (check == NULL)
-				check = g_find_program_in_path ("color-xterm");
-			if (check == NULL)
-				check = g_find_program_in_path ("rxvt");
-			if (check == NULL)
-				check = g_find_program_in_path ("xterm");
-			if (check == NULL)
-				check = g_find_program_in_path ("dtterm");
-			if (check == NULL) {
-				g_warning (_("Cannot find a terminal, using "
-					     "xterm, even if it may not work"));
-				check = g_strdup ("xterm");
-			}
-			terminal = check;
-			exec_flag = g_strdup ("-e");
-		}
-	}
-
-	if (terminal == NULL)
-		return NULL;
-
-	result = g_strconcat (terminal, " ", exec_flag, NULL);
-	return result;
-}
-
-
-gchar *
-application_get_command (const GnomeVFSMimeApplication *app)
-{
-	char *command;
-	const char *bad_chars = "$\'`\"\\!?*()[]&|@#:;"; /* similar to shell_escape but without ' ' */
-	
-	if (app->requires_terminal) {
-		char *terminal;
-		char *command_to_exec;
-
-		terminal = get_terminal ();
-		if (terminal == NULL)
-			return NULL;
-
-		command_to_exec = escape_str (app->command, bad_chars);
-		command = g_strconcat (terminal,
-				       " ",
-				       command_to_exec,
-				       NULL);
-		g_free (terminal);
-		g_free (command_to_exec);
-	} else
-		command = escape_str (app->command, bad_chars);
-
-	return command;
-}
-
-
 gboolean
 match_patterns (char       **patterns, 
 		const char  *string,
@@ -941,6 +865,8 @@ _g_strdup_with_max_size (const char *s,
 const char *
 eat_spaces (const char *line)
 {
+	if (line == NULL)
+		return NULL;
 	while ((*line == ' ') && (*line != 0))
 		line++;
 	return line;
@@ -960,6 +886,10 @@ split_line (const char *line,
 
 	scan = eat_spaces (line);
 	for (i = 0; i < n_fields; i++) {
+		if (scan == NULL) {
+			fields[i] = NULL;
+			continue;
+		}
 		field_end = strchr (scan, ' ');
 		if (field_end != NULL) {
 			fields[i] = g_strndup (scan, field_end - scan);
@@ -978,9 +908,14 @@ get_last_field (const char *line,
 	const char *field;
 	int         i;
 
+	if (line == NULL)
+		return NULL;
+
 	last_field--;
 	field = eat_spaces (line);
 	for (i = 0; i < last_field; i++) {
+		if (field == NULL)
+			return NULL;
 		field = strchr (field, ' ');
 		field = eat_spaces (field);
 	}
@@ -998,29 +933,6 @@ get_temp_work_dir (void)
 }
 
 
-#define MAX_TRIES 50
-
-
-char *
-get_temp_work_dir_name (void)
-{
-	char       *result = NULL;
-	static int  count = 0;
-	int         try = 0;
-
-	do {
-		g_free (result);
-		result = g_strdup_printf ("%s%s.%d.%d",
-					  g_get_tmp_dir (),
-					  "/file-roller",
-					  getpid (),
-					  count++);
-	} while (path_is_file (result) && (try++ < MAX_TRIES));
-
-	return result;
-}
-
-
 /* file list utils */
 
 
@@ -1165,3 +1077,97 @@ str_substitute (const char *str,
 
 	return g_string_free (gstr, FALSE);
 }
+
+
+char *
+escape_uri (const char *uri)
+{
+	const char *start = NULL;
+	const char *uri_no_method;
+	char       *method;
+	char       *epath, *euri;
+
+	if (uri == NULL)
+		return NULL;
+
+	start = strstr (uri, "://");
+	if (start != NULL) {
+		uri_no_method = start + strlen ("://");
+		method = g_strndup (uri, start - uri);
+	} else {
+		uri_no_method = uri;
+		method = NULL;
+	}
+	
+	epath = gnome_vfs_escape_host_and_path_string (uri_no_method);
+	
+	if (method != NULL) {
+		euri = g_strdup_printf ("%s://%s", method, epath);
+		g_free (epath);
+	} else
+		euri = epath;
+
+	g_free (method);
+
+	return euri;
+}
+
+
+gboolean
+check_permissions (const char *path,
+		   int         mode) 
+{
+	GnomeVFSFileInfo *info;
+	GnomeVFSResult    vfs_result;
+	char             *escaped;
+
+	info = gnome_vfs_file_info_new ();
+	escaped = escape_uri (path);
+	vfs_result = gnome_vfs_get_file_info (escaped, 
+					      info, 
+					      (GNOME_VFS_FILE_INFO_DEFAULT 
+					       | GNOME_VFS_FILE_INFO_FOLLOW_LINKS
+					       | GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS));
+	g_free (escaped);
+
+	if (vfs_result != GNOME_VFS_OK) 
+		return FALSE; 
+
+	if ((mode & R_OK) && ! (info->permissions & GNOME_VFS_PERM_ACCESS_READABLE)) 
+		return FALSE; 
+
+	if ((mode & W_OK) && ! (info->permissions & GNOME_VFS_PERM_ACCESS_WRITABLE)) 
+		return FALSE; 
+
+	if ((mode & X_OK) && ! (info->permissions & GNOME_VFS_PERM_ACCESS_WRITABLE)) 
+		return FALSE; 
+
+	return TRUE;
+}
+
+
+gboolean 
+is_program_in_path (const char *filename)
+{
+        char *str;
+	char *value;
+	int   result = FALSE;
+
+	value = g_hash_table_lookup (programs_cache, filename);
+	if (value != NULL) {
+		result = (strcmp (value, "1") == 0);
+		return result;
+	}
+
+        str = g_find_program_in_path (filename);
+        if (str != NULL) {
+                g_free (str);
+		result = TRUE;
+        }
+
+	g_hash_table_insert (programs_cache, 
+			     g_strdup (filename), 
+			     result ? "1" : "0");
+	
+        return result;
+}
